/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx.h"
#include <stdio.h>
#include <string.h>
#include "nrf24.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define RX_PAYLOAD           19   // nRF24L01 RX payload length
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile uint32_t msTicks;
uint16_t TimerPeriod = 0;
uint16_t Channel1Pulse = 0, Channel2Pulse = 0, Channel3Pulse = 0, Channel4Pulse = 0;
uint16_t RED_value = 0, GREEN_value = 0, BLUE_value = 0;
uint16_t readyToWrite = 0, interval = 0;

uint8_t q = 0;
uint8_t data_array[5];
uint8_t tx_address[5] = {0xD7,0xD7,0xD7,0xD7,0xD7};
uint8_t rx_address[5] = {0xE7,0xE7,0xE7,0xE7,0xE7};

union four_byte{
	int8_t b[4];
	float fl_t;
	int16_t do_t[2];
	uint16_t unsigInt16_t[2];
	int32_t lo_t;
	uint32_t unsigInt32_t;
} foby;

/* Private function prototypes -----------------------------------------------*/
int fputc(int ch, FILE * f);
void SysTick_Handler(void);
void Delay (uint32_t dlyTicks);
void GPIO_Config(void);
void Serial_Config(void);
void PWM_Config(void);
void updateRGB(uint16_t rVal, uint16_t gVal, uint16_t bVal);
void fadeToColor(uint16_t rVal, uint16_t gVal, uint16_t bVal, uint16_t milliSecs);

/* Private functions ---------------------------------------------------------*/

/*
  retargeting output of printf() to serial line
*/
/* Write the 'ch' character into the 'f' file */
  int fputc(int ch, FILE * f) {
  /* Transmit the character using USART1 */
	USART_SendData(USART1, (uint8_t) ch);

  /* Wait until transmit finishes */
	while (USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET);

	return ch;
  }

/*
  SysTick handler
 */
  void SysTick_Handler(void) {
	msTicks++;
  }

/*
  Function to provide delay (in mSec)
 */
  void Delay (uint32_t delayTicks) {                                              
	uint32_t currentTicks;

	currentTicks = msTicks;
	while ((msTicks - currentTicks) < delayTicks);
  }

/**
  * @brief  Main program.
  * @param  None
  * @retval None
*/
int main(void)
{
	SystemCoreClockUpdate();                      	/* Get Core Clock Frequency   */
	if (SysTick_Config(SystemCoreClock / 1000)) 	/* SysTick 1 msec interrupts  */
	{ 												/* Capture error              */
		while (1);                                  
	}

	RED_value = 0;
	GREEN_value = 0;
	BLUE_value = 0;
	interval = 1000;

	GPIO_Config();
	PWM_Config();
	Serial_Config();
	
	/* init nrf24 hardware pins */
	nrf24_init();
  
	/* Channel #2 , payload length: 5 */
	nrf24_config(2,5);
  
	/* Set the device addresses */
	nrf24_tx_address(tx_address);
	nrf24_rx_address(rx_address);

	printf("Hardware Init complete");
	printf("\r\n");

	while(1)
	{
		fadeToColor(0x0000, 0x0000, 0x0000, 1000);
		Delay(500);
		fadeToColor(0x0000, 0x0000, 0xFFFF, 1000);
		Delay(500);
		fadeToColor(0x0000, 0xFFFF, 0x0000, 1000);
		Delay(500);
		fadeToColor(0x0000, 0xFFFF, 0xFFFF, 1000);
		Delay(500);
		fadeToColor(0xFFFF, 0x0000, 0x0000, 1000);
		Delay(500);
		fadeToColor(0xFFFF, 0x0000, 0xFFFF, 1000);
		Delay(500);
		fadeToColor(0xFFFF, 0xFFFF, 0x0000, 1000);
		Delay(500);
		fadeToColor(0xFFFF, 0xFFFF, 0xFFFF, 1000);
		Delay(500);
	}

	/*
	while (1)
	{
		if(nrf24_dataReady())
		{
			nrf24_getData(data_array); 
			
			foby.b[0] = data_array[1];
			foby.b[1] = data_array[2];
			foby.b[2] = data_array[3];
			foby.b[3] = data_array[4];
			
			switch(data_array[0]){
				case 0x01:
					RED_value = (uint16_t) foby.do_t[0];
					break;
				case 0x02:
					GREEN_value = (uint16_t) foby.do_t[0];
					break;
				case 0x03:
					BLUE_value = (uint16_t) foby.do_t[0];
					break;
				case 0x04:
					interval = (uint16_t) foby.do_t[0];
					readyToWrite = 1;
					break;
				default:
					//GPIO_SetBits(GPIOC, GPIO_Pin_9);
					//Delay(5);
					//GPIO_ResetBits(GPIOC, GPIO_Pin_9);
					break;
			}
	
			if(readyToWrite == 1)
			{
				updateRGB(RED_value, GREEN_value, BLUE_value);
				readyToWrite = 0;
			}
		}
	}
	*/
	return(0);
}

void GPIO_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStruct;

	/* GPIOC config - LED Configuration */
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOC, ENABLE );
	GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
	
	/* MOTOR GPIO config */
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE );
	
	GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_4;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init( GPIOA, &GPIO_InitStruct );

	/* BUTTON */
	GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_0;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStruct);

	GPIO_SetBits(GPIOC, GPIO_Pin_8);
	GPIO_SetBits(GPIOC, GPIO_Pin_9);
}

void Serial_Config(void)
{
	GPIO_InitTypeDef port_init;
	USART_InitTypeDef usart_init;
	
	/*Enable or disable the AHB peripheral clock */
  	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

  	/*Configure GPIO pin : PB */
  	port_init.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
  	port_init.GPIO_Mode = GPIO_Mode_AF;
  	port_init.GPIO_OType = GPIO_OType_PP;
  	port_init.GPIO_PuPd = GPIO_PuPd_UP;
  	port_init.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOB, &port_init);

  	/*Configure GPIO pin alternate function */
  	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_0);

  	/*Configure GPIO pin alternate function */
  	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_0);

	//USART init
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
	usart_init.USART_BaudRate = 115200;
	usart_init.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	usart_init.USART_Parity = USART_Parity_No;
	usart_init.USART_StopBits = USART_StopBits_1;
	usart_init.USART_WordLength = USART_WordLength_8b;
	usart_init.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	USART_Init(USART1, &usart_init);
	USART_Cmd(USART1, ENABLE);

}

void fadeToColor(uint16_t rVal, uint16_t gVal, uint16_t bVal, uint16_t milliSecs)
{
	int32_t redStep, greenStep, blueStep;
	uint16_t index;

	redStep = (rVal - RED_value) / milliSecs;
	greenStep = (gVal - GREEN_value) / milliSecs;
	blueStep = (bVal - BLUE_value) / milliSecs;

	for(index = 0; index <= milliSecs; index++)
	{
		updateRGB(RED_value + (redStep)*index, GREEN_value + (greenStep)*index, BLUE_value + (blueStep)*index);
		Delay(1);
	}

	updateRGB(rVal, gVal, bVal);
	Delay(1);

	RED_value = rVal;
	GREEN_value = gVal;
	BLUE_value = bVal;

}

void updateRGB(uint16_t rVal, uint16_t gVal, uint16_t bVal)
{
  /* Compute the value to be set in ARR regiter to generate signal frequency at 10.00 Khz */
  TimerPeriod = (SystemCoreClock / 10000 ) - 1;
  /* Compute CCR1 value to generate a duty cycle at rVal%  for channel 1 */
  Channel1Pulse = (uint16_t) (((uint32_t) rVal * (TimerPeriod - 1)) / 0xFFFF);
  /* Compute CCR2 value to generate a duty cycle at gVal%  for channel 2 */
  Channel2Pulse = (uint16_t) (((uint32_t) gVal * (TimerPeriod - 1)) / 0xFFFF);
  /* Compute CCR3 value to generate a duty cycle at bVal%  for channel 3 */
  Channel3Pulse = (uint16_t) (((uint32_t) bVal * (TimerPeriod - 1)) / 0xFFFF);

  TIM1->CCR1 = Channel1Pulse;
  TIM1->CCR2 = Channel2Pulse;
  TIM1->CCR3 = Channel3Pulse;
}

void PWM_Config(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;

  /* GPIOA, GPIOA Clocks enable */
  RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE);
  
  /* GPIOA Configuration: Channel 1, 2, 3, 4 as alternate function push-pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_2);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_2);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_2);
  
  /* TIM1 Configuration ---------------------------------------------------
   Generate 3 PWM signals with 4 different duty cycles:
   TIM1 input clock (TIM1CLK) is set to APB2 clock (PCLK2)    
	=> TIM1CLK = PCLK2 = SystemCoreClock
   TIM1CLK = SystemCoreClock, Prescaler = 0, TIM1 counter clock = SystemCoreClock
   SystemCoreClock is set to 48 MHz for STM32F0xx devices
   
   The objective is to generate 3 PWM signal at 17.57 KHz:
	 - TIM1_Period = (SystemCoreClock / 17570) - 1
   The channel 1 duty cycle is set to 50%
   The channel 2 duty cycle is set to 37.5%
   The channel 3 duty cycle is set to 25%
   The Timer pulse is calculated as follows:
	 - ChannelxPulse = DutyCycle * (TIM1_Period - 1) / 100
   
   Note: 
	SystemCoreClock variable holds HCLK frequency and is defined in system_stm32f0xx.c file.
	Each time the core clock (HCLK) changes, user had to call SystemCoreClockUpdate()
	function to update SystemCoreClock variable value. Otherwise, any configuration
	based on this variable will be incorrect. 
  ----------------------------------------------------------------------- */
  /* Compute the value to be set in ARR regiter to generate signal frequency at 10.00 Khz */
  TimerPeriod = (SystemCoreClock / 10000 ) - 1;
  /* Compute CCR1 value to generate a duty cycle at 50% for channel 1 */
  // Channel1Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 10);
  /* Compute CCR2 value to generate a duty cycle at 50%  for channel 2 */
  // Channel2Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 10);
  /* Compute CCR3 value to generate a duty cycle at 50%  for channel 3 */
  // Channel3Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 10);

  /* TIM1 clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 , ENABLE);
  
  /* Time Base configuration */
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  /* Channel 1, 2,3 and 4 Configuration in PWM mode */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;

  TIM_OC1Init(TIM1, &TIM_OCInitStructure);

  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OC2Init(TIM1, &TIM_OCInitStructure);

  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OC3Init(TIM1, &TIM_OCInitStructure);

  /* TIM1 counter enable */
  TIM_Cmd(TIM1, ENABLE);

  /* TIM1 Main Output Enable */
  TIM_CtrlPWMOutputs(TIM1, ENABLE);
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
  void assert_failed(uint8_t* file, uint32_t line)
  {
  /* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
	while (1)
	{
	}
  }
#endif
